unit UEuclid;

interface

uses
  System.SysUtils, WinAPI.Windows;

type
  TEuclid = class
  type TStatusMessage = (stmABzero,stmABCreduced,stmNoSolution,stmABCtoobig,
                         stmSolution);
  const statusmessage  : array[TStatusmessage] of string =
                        ('A,B can not be 0',
                         'A,B,C reduced',
                         'no solution for this numbers',
                         'numbers too big',
                         'solution found');

  strict private
    fA: integer;
    fB: integer;
    fC: integer;

    fXDelta: integer;
    fYDelta: integer;

    fX: integer;
    fY: integer;

  public
    function Calculate(aA,aB,aC: integer): TStatusMessage;

    property AValue: integer read fA;
    property BValue: integer read fB;
    property CValue: integer read fC;

    property XDelta: integer read fXDelta;
    property YDelta: integer read fYDelta;

    property XValue: integer read fX;
    property YValue: integer read fY;

    procedure Previous;
    procedure Successor;

  end;

implementation

{ TEuclid }

function TEuclid.Calculate(aA,aB,aC: integer): TStatusMessage;
const
  QueueMax = 10;
var
  hvalue, GCD: integer;
  Amin, Bmin, Cmin, ABswap: boolean;
  Queue: array [1..QueueMax] of integer;
  i, r, f, A, B, C, Xcomp, Ycomp: integer;
begin
  fA := aA;
  fB := aB;
  fC := aC;
  fXDelta := 0;
  fYDelta := 0;

  if (fA = 0) or (fB = 0) then
    begin
     result := stmABZero;
     exit;
    end;

  Amin := (fA < 0);
  Bmin := (fB < 0);
  Cmin := (fC < 0);

  A := abs(fA);
  B := abs(fB);
  C := abs(fC);

  abswap := A < B;
  if abswap then
    begin
      hvalue := A;
      A := B;
      B := hvalue;
    end;

  for i := low(Queue) to high(Queue) do Queue[i] := 0; //reset quotient table

  i := 0;
  repeat                          // calculate GCD and quotient table
    i := i + 1;
    r := A mod B;
    Queue[i] := -(A div B);
    A := B;
    B := r;
  until r = 0;

  GCD := A;
  if GCD <> 1 then
   begin
    if (fC mod GCD = 0) then
      begin
        fA := fA div GCD;
        fB := fB div GCD;
        fC := fC div GCD;
        result := stmABCreduced;
      end
    else result := stmNoSolution;
    exit;
   end;

  Queue[i] := 1;                  // backward substitution
    if i > 2 then
      for i := i - 2 downto 1 do
        Queue[i] := Queue[i + 2] + Queue[i + 1] * Queue[i];

  if abswap then                 // reload A,B
    begin
      A := abs(fB);
      B := abs(fA);
    end
    else begin
          A := abs(fA);
          B := abs(fB);
         end;

    Xcomp := 0;
    Ycomp := 0;
    if C >= (A + B) then
    begin                         // C reduction
      f := C div (A + B);
      C := C mod (A + B);
      Xcomp := Xcomp + f;
      Ycomp := Ycomp + f;
    end;

    if C >= A then
    begin
      f := C div A;
      C := C mod A;
      Xcomp := Xcomp + f;
    end;

    if C >= B then
    begin
      f := C div B;
      C := C mod B;
      Ycomp := Ycomp + f;
    end;

    if (A - B) <> 0 then
      if C >= (A - B) then
      begin
        f := C div (A - B);
        C := C mod (A - B);
        Xcomp := Xcomp + f;
        Ycomp := Ycomp - f;
      end;

    try
      fY := Queue[1] * C + Ycomp;
      fX := Queue[2] * C + Xcomp;
    except
      Result := stmABCTooBig;
      exit;
    end;

    // correction for unnecessary big |X| , |Y| (C > 1)

    while (abs(fX - B) + abs(fY + A)) < (abs(fX) + abs(fY)) do
    begin
      fX := fX - B;
      fY := fY + A;
    end;

    while (abs(fX + B) + abs(fY - A)) < (abs(fX) + abs(fY)) do
    begin
      fX := fX + B;
      fY := fY - A;
    end;

    // correction negative original A, B, or A < B swap

    if abswap then
    begin
      hvalue := fX;
      fX := fY;
      fY := hvalue;
    end;

    if Amin then fX := -fX;
    if Bmin then fY := -fY;
    if Cmin then begin
                  fX := -fX;
                  fY := -fY;
                 end;

    fXDelta := fB;
    fYDelta := -fA;
    Result := stmSolution;
end;

procedure TEuclid.Previous;
begin
  dec(fX, fXDelta);
  dec(fY, fYDelta);
end;

procedure TEuclid.Successor;
begin
  inc(fX, fXDelta);
  inc(fY, fYDelta);
end;

end.
